/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*==========================================================================*/
/* Module Name: MQUTILS.C                                                   */
/*==========================================================================*/

/* Include standard libraries */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>


/* Include Application headers */
#include "saveqmgr.h"

extern int convert;
void  StripTrailingBlanks( char * , char * , int );
void  DoubleUpQuotes(char * , char * );
void  InsertNewlines ( char * , char *  );


MQLONG OpenQ( MQHCONN     hConn
            , MQCHAR48    QMgrName
            , MQCHAR48    QName
            , MQCHAR48   *DynamicQName
            , MQLONG      OpenOpts
            , MQLONG     *pHOpenQ
            )
{
   MQLONG                 CompCode = 0;
   MQLONG                 Reason = 0;
   MQOD                   ObjDesc = {MQOD_DEFAULT};

   /* ******************************************************************** */
   /* If QName specifies the name of a model queue then we are going to    */
   /* open a dynamic queue, the properties of which are specified by the   */
   /* model queue definition. DynamicQName then represents the name to be  */
   /* given to the dynamic queue. If there is an asterisk anywhere in the  */
   /* first 33 character positions it will be replaced by a generated a    */
   /* unique name. The actual name is returned in place of the model       */
   /* queue used and is stored for future use.                             */
   /* ******************************************************************** */

   memcpy( ObjDesc.ObjectName, QName, MQ_Q_NAME_LENGTH );
   memcpy( ObjDesc.ObjectQMgrName, QMgrName, MQ_Q_MGR_NAME_LENGTH );
   if ( DynamicQName != NULL ) {
      memcpy( ObjDesc.DynamicQName, *DynamicQName, MQ_Q_NAME_LENGTH );
   } /* endif */

   MQOPEN(  hConn                     /* I  : Queue manager handle   */
         , &ObjDesc                   /* IO : queue attributes       */
         ,  OpenOpts                  /* I  :                        */
         ,  pHOpenQ                    /*  O : handle to open queue   */
         , &CompCode                  /*  O : OK/Warning/Error       */
         , &Reason                    /*  O : Reason for above       */
         );

   if (CompCode != MQCC_OK ) {
      fprintf(stderr,  "MQOPEN failed for %s, CC=%ld RC=%ld\n"
            , QName
            , CompCode
            , Reason
            );
   } /* endif */

   /* --------------------------------------------------------------- */
   /* If we have opened a dynamic queue then the actual name supplied */
   /* has been returned in the object descriptor in place of the      */
   /* supplied model queue name                                       */
   /* --------------------------------------------------------------- */
   if ( DynamicQName != NULL ) {
      memcpy( *DynamicQName, ObjDesc.ObjectName, MQ_Q_NAME_LENGTH );
   } /* endif */

   return( CompCode );

}




MQLONG PutMsg( MQHCONN   hConn       /* Connection to queue manager            */
           , MQCHAR8   MsgFormat   /* Format of user data to be put in msg   */
           , MQHOBJ    hQName      /* handle of queue to put the message to  */
           , MQCHAR48  QName       /* name of queue to put the message to    */
           , MQCHAR48  ReplyToQ    /* name of q on which to receive replies  */
           , MQBYTE   *UserMsg     /* The user data to be put in the message */
           , MQLONG    UserMsgLen  /*                                        */
           )

{
   MQMD    MsgDesc = {MQMD_DEFAULT};      /* Message description        */
   MQPMO   PutMsgOpts = {MQPMO_DEFAULT};  /* Controls action of MQPUT   */
   MQLONG  CompCode;                      /* Completion code            */
   MQLONG  Reason;                        /* reason qualifying CompCode */


   /********************************/
   /* Set up message header fields */
   /********************************/

   MsgDesc.Persistence    = MQPER_NOT_PERSISTENT;
   memset( MsgDesc.ReplyToQMgr, '\0', MQ_Q_MGR_NAME_LENGTH );
   MsgDesc.MsgType  = MQMT_REQUEST;
   memcpy( MsgDesc.ReplyToQ, ReplyToQ, MQ_Q_NAME_LENGTH );
   memcpy( MsgDesc.Format,   MsgFormat, MQ_FORMAT_LENGTH );

   MsgDesc.Expiry = 1800;  /* 3 minutes */

   PutMsgOpts.Options  = MQPMO_NO_SYNCPOINT;


   MQPUT(  hConn                     /* I  : Queue manager handle     */
        ,  hQName                    /* I  : Queue handle             */
        , &MsgDesc                   /* IO : Message attributes       */
        , &PutMsgOpts                /* IO : Control action of MQPUT  */
        ,  UserMsgLen                /* I  : User message length      */
        ,  UserMsg                   /* I  : User message             */
        , &CompCode                  /*  O : OK/Warning/Error         */
        , &Reason                    /*  O : Reason for above         */
        );

   if ( CompCode ) {
      fprintf(stderr, "MQPUT failed for queue %s. CC=%ld RC=%ld\n"
            , QName
            , CompCode
            , Reason
            );

   } /* endif */

   return (CompCode);

} /* end PutMsg */



MQLONG GetMsg( MQHCONN   hConn          /* handle of queue manager            */
           , MQHOBJ    hQName         /* handle of queue to read from       */
           , MQBYTE  **UserMsg        /* Input/Output buffer containing msg */
           , MQLONG   *ReadBufferLen  /* Length of supplied buffer          */
           )
{
   MQMD    MsgDesc = {MQMD_DEFAULT};       /* Message description           */
   MQGMO   GetMsgOpts = {MQGMO_DEFAULT};   /* Options to control MQGET      */
   MQLONG  UserMsgLen;                     /* Actual length received        */
   MQLONG  CompCode;                       /* Completion code               */
   MQLONG  Reason;                         /* reason qualifying CompCode    */



   if (convert) {
     GetMsgOpts.Options  = MQGMO_WAIT + MQGMO_CONVERT + MQGMO_FAIL_IF_QUIESCING;
   } else {
     GetMsgOpts.Options  = MQGMO_WAIT + MQGMO_FAIL_IF_QUIESCING;
   }
   GetMsgOpts.WaitInterval  = WAIT_INT;

   /* [v520] on Tandem NSK, you MUST use the no syncpoint option on mqget */
   #if defined(_OSS_TARGET) || defined(_GUARDIAN_TARGET)
       GetMsgOpts.Options += MQGMO_NO_SYNCPOINT;
   #endif

   do  {

   MQGET(  hConn            /* I  : Queue manager handle              */
        ,  hQName           /* I  : handle to QName to read from      */
        , &MsgDesc          /* IO : message attributes                */
        , &GetMsgOpts       /* IO : control action of MQGET           */
        , *ReadBufferLen    /* I  : length of supplied buffer         */
        , *UserMsg          /*  O : User data to be put               */
        , &UserMsgLen       /*  O : Actual length of returned message */
        , &CompCode         /*  O : OK/Warning/Error                  */
        , &Reason           /*  O : Reason for above                  */
        );

        if (Reason==MQRC_TRUNCATED_MSG_FAILED) {
          if (*UserMsg) free(*UserMsg);
          *ReadBufferLen = UserMsgLen;
          if (!(*UserMsg=malloc(*ReadBufferLen))){
            fprintf(stderr, "Failed to allocate %ld bytes\n",*ReadBufferLen);
            CompCode = MQCC_FAILED;
            Reason = MQRC_ENVIRONMENT_ERROR;
            }/* End if*/
          }/* End if*/

     } while (Reason==MQRC_TRUNCATED_MSG_FAILED);/* End do*/

   if ( CompCode ) {
     if (Reason == MQRC_NO_MSG_AVAILABLE)
         fprintf(stderr,
      "No reply msg, make sure command server is started (strmqcsv qmgr-name)\n");
     else
         fprintf(stderr,  "MQGET failed for reply queue. MQCC=%ld MQRC=%ld\n"
            , CompCode
            , Reason
            );
   }

    return (CompCode);

}



/* ------------------------------------------------------------------------ */
/* MQsprintf is used to write the value of the string attributes to our     */
/* file.                                                                    */
/*                                                                          */
/* The text of the insert is exactly as it was returned by the queue        */
/* manager during processing by the command server. That is, a string       */
/* of characters which indicate the value of a specific attribute, padded   */
/* with blanks. We will change these blanks to nulls.                       */
/*                                                                          */
/* It is possible that the text to be written, for instance a queue         */
/* name, is the maximum length for the object concerned. In this case the   */
/* string is not null terminated so sprintf would not behave as we intend.  */
/* For this reason, MQsprintf assumes only one text insert and takes an     */
/* additional parameter indicating the maximum length the the particular    */
/* attribute being written.                                                 */
/* ------------------------------------------------------------------------ */

/* The "classic" MQsprintf which does it all */

void MQsprintf( char    *ParmBuffer
              , char    *Format
              , char    *Insert
              , int      Length
              )
{
  MQsprintf_X(      ParmBuffer
                , Format
                , Insert
                , Length
                ,MQSPRINTF_X_LINE_BREAKS
                + MQSPRINTF_X_DOUBLE_QUOTES
                + MQSPRINTF_X_NO_TRAILING_BLANKS
                ) ;
}


/* [v530] The "new" MQsprintf which accepts options */

void MQsprintf_X (char    *ParmBuffer
              , char    *Format
              , char    *Insert
              , int      Length
              , int      Options
              )
{
   char     *MQParm = NULL;
   char     *MQParm2 = NULL;
   int      mlen     = 0;

   /* Allocate a temporary buffer larger than the               */
   /* maximum allowed for this attribute. Initialise to nulls.  */
   /* This will ensure that the buffer is null terminated when  */
   /* subsequently passed to sprintf.                           */

   mlen = 2 * (strlen(Format) + Length);
   if(!(MQParm = (char *)malloc( mlen )) ||
      !(MQParm2 = (char *)malloc( mlen ))) {
     fprintf(stderr, "Error allocating %d bytes in MQsprintf\n", mlen );
     exit(2);
   }
   memset( MQParm, '\0', mlen );
   memset( MQParm2, '\0', mlen );


   if (Options & MQSPRINTF_X_NO_TRAILING_BLANKS)
        StripTrailingBlanks( MQParm, Insert, Length);
   else
        strncpy( MQParm, Insert, Length );

   /* Now, if the string is null, make it a single blank */

   if (MQParm[0]=='\0')
     MQParm[0] = ' ';


   if (Options & MQSPRINTF_X_DOUBLE_QUOTES)
        DoubleUpQuotes(MQParm2, MQParm);
   else
        strcpy( MQParm2, MQParm );

   /* Format the string  */
   sprintf( MQParm , Format , MQParm2 );

   if (Options & MQSPRINTF_X_LINE_BREAKS       )
        InsertNewlines ( ParmBuffer, MQParm );
   else
        strcpy(ParmBuffer, MQParm);

   free (MQParm);
   free (MQParm2);

}

/*****************************************************************************/
/* [v512] wms                                                                */
/* This routine converts a string list into a string of the form:            */
/* 'exit1', 'exit2', 'exit3'                                                 */
/*****************************************************************************/

void   makeStringFromList( char* targetString,
                           char * sourceString,
                           MQLONG stringLength,
                           MQLONG stringCount ) {

#define STREND "', +\n          '"
#define STRENDLEN strlen(STREND)

  int i = 0;
  char * pT = NULL;
  char * pS = NULL;

  pS = sourceString;
  pT = targetString;

  if (stringCount) {
     strcpy(pT, "'");  /* opening quote */
     pT++;
     for ( i = 0; i< stringCount; i++) {
       strncat(pT, pS, stringLength);
       strcat(pT, STREND );
       pS+=stringLength;
     }
     strcat(pT, "'");   /* closing quote */
  } else {
    strcpy(pT, " ");
  }

  /* trim off the ending quotes and comma */
  if (strlen(pT) >= STRENDLEN)
  *(pT + strlen(pT) - STRENDLEN) = '\0';

  return;

}


/* [v530] small rtn to Double up single quotes  */

void   DoubleUpQuotes(char * MQParm2, char * MQParm) {

   /* Go through the string, */
   /* changing any single quotes to two single quotes */
   /* Copying to MQParm2 as we go along */
   int scounter = 0;
   int tcounter = 0;
   for (scounter = 0; scounter < strlen(MQParm); scounter++){
     MQParm2[tcounter++] = MQParm[scounter];
     if (MQParm2[tcounter-1]=='\'')
       MQParm2[tcounter++]='\'';
   }
}

/* [v530] Remove trailing blanks */

void StripTrailingBlanks( char * MQParm, char * Insert, int Length){

   int scounter = Length - 1;
   /* Scan backwards until we find a non-blank character */
   while (scounter >= 0 && Insert[scounter] == ' ')
       scounter--;
   /* Copy the original INSERT string into MQParm buffer */
   while ( scounter >= 0) {
       MQParm[scounter] = Insert[scounter];
       scounter--;
   } /* endwhile */

}

/* [v530] */
/* We need to assure that no lines are more than 70 characters  ... */
/* The following loop newlines the string into  */
/* maximum 70 character chunks */
/* Copy into the Return ParmBuffer area as we go along */

void InsertNewlines ( char * ParmBuffer, char * MQParm ) {

  #define LINEBREAKEND   "-\n"
   int curlen = 0;
   int tcounter = 0;
   int scounter = 0;

   for (scounter=0; scounter < strlen(MQParm); scounter++) {
      ParmBuffer[tcounter++] = MQParm[scounter];
      curlen++;
      /* if we are processing a string list, they'll have newlines in them */
      if (ParmBuffer[tcounter]=='\n')
        curlen = 0;
      else if (!(curlen % (70-strlen(LINEBREAKEND))) &&
               (strlen(&MQParm[scounter]) > strlen(LINEBREAKEND))
            ) {
        ParmBuffer[tcounter] = '\0';
        strcat(ParmBuffer, LINEBREAKEND);
        tcounter += strlen(LINEBREAKEND);
        curlen = 0;
      }  /* end if !curlen ... */
   } /* End for*/

  /* Null terminate the buffer we are returing */
  ParmBuffer[tcounter] = '\0';

}
