/*===========================================================================*/
/*                                                                           */
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Sterling Forest, NY, USA                                     */
/*                                                                           */
/*              Chris Petty is the original author                           */
/*                                                                           */
/* (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003    */
/*===========================================================================*/

/*===========================================================================*/
/* Module Name: SAVEQMGR.H                                                   */
/*===========================================================================*/

#define THISVERSION "SAVEQMGR V5.3.1"

/* [WMS] Using predefined constants instead of hard coding numbers for */
/* string sizes.                                                       */

/* defining these command levels removes the need for a lot of */
/* conditional testing in the source code                      */


#ifndef MQCMDL_LEVEL_1
 #define MQCMDL_LEVEL_1   100L
#endif

#ifndef MQCMDL_LEVEL_101
   #define MQCMDL_LEVEL_101 101L
#endif

#ifndef MQCMDL_LEVEL_110
 #define MQCMDL_LEVEL_110 110L
#endif

#ifndef MQCMDL_LEVEL_114
 #define MQCMDL_LEVEL_114 114L
#endif

#ifndef MQCMDL_LEVEL_120
 #define MQCMDL_LEVEL_120 120L
#endif

#ifndef MQCMDL_LEVEL_200
 #define MQCMDL_LEVEL_200 200L
#endif

#ifndef MQCMDL_LEVEL_201
 #define MQCMDL_LEVEL_201 201L
#endif

#ifndef MQCMDL_LEVEL_210
 #define MQCMDL_LEVEL_210 210L
#endif

#ifndef MQCMDL_LEVEL_220
 #define MQCMDL_LEVEL_220 220L
#endif

#ifndef MQCMDL_LEVEL_221
 #define MQCMDL_LEVEL_221 221L
#endif

#ifndef MQCMDL_LEVEL_320
 #define MQCMDL_LEVEL_320 320L
#endif

#ifndef MQCMDL_LEVEL_420
 #define MQCMDL_LEVEL_420 420L
#endif

#ifndef MQCMDL_LEVEL_500
 #define MQCMDL_LEVEL_500 500L
#endif

#ifndef MQCMDL_LEVEL_510
 #define MQCMDL_LEVEL_510 510L
#endif

/* Note, we don't define MQCMDL_LEVEL_520 because there are no */
/* attributes introduced in v5.2 which we can used to determine */
/* the "compiled" level of the program.  Besides, we don't use */
/* it in any other preprocessor directives */

#ifndef MQCMDL_LEVEL_530
 #define MQCMDL_LEVEL_530 530L
#endif


/* [v520] Included for Tandem NSK changes */
#ifndef MQ_TOTAL_EXIT_DATA_LENGTH
#define MQ_TOTAL_EXIT_DATA_LENGTH 999L
#endif
#ifndef MQ_TOTAL_EXIT_NAME_LENGTH
#define MQ_TOTAL_EXIT_NAME_LENGTH 999L
#endif

/* V32 special command type to get clntconn channels */
#define XZCMD_INQUIRE_CHANNEL_CLNT MQMT_APPL_FIRST+1

/* V513 get the right length for MCAUserId field */
#if defined(MQ_MAX_MCA_USER_ID_LENGTH)
   #define XZMCAUSERLEN MQ_MAX_MCA_USER_ID_LENGTH
#else
   #define XZMCAUSERLEN MQ_USER_ID_LENGTH
#endif


typedef struct QMgrParms {
   MQCHAR      QMgrName[MQ_Q_MGR_NAME_LENGTH];
   MQCHAR      QMgrDesc[MQ_Q_MGR_DESC_LENGTH];
   MQLONG      Platform;
   MQLONG      CommandLevel;        /*[wms]level 500 indicates V5 etc */
   MQLONG      TriggerInterval;
   MQCHAR      DeadLetterQName[MQ_Q_NAME_LENGTH];
   MQLONG      MaxPriority;
   MQCHAR      CommandInputQName[MQ_Q_NAME_LENGTH];
   MQCHAR      DefXmitQName[MQ_Q_NAME_LENGTH];
   MQLONG      CodedCharSetId;
   MQLONG      MaxHandles;
   MQLONG      MaxUncommittedMsgs;
   MQLONG      MaxMsgLength;
   MQLONG      SyncPoint;
   MQLONG      AuthorityEvent;
   MQLONG      InhibitEvent;
   MQLONG      LocalEvent;
   MQLONG      RemoteEvent;
   MQLONG      StartStopEvent;
   MQLONG      PerformanceEvent;

   MQLONG      DistLists;           /* [wms] V5 */
   MQLONG      ChannelAutoDef;      /* [wms] V5 */
   MQLONG      ChannelAutoDefEvent; /* [wms] V5 */
   MQCHAR      ChannelAutoDefExit[MQ_EXIT_NAME_LENGTH];        /* [wms] V5 */

   MQCHAR      WorkLoadExit[MQ_EXIT_NAME_LENGTH];              /* [v51] V51 */
   MQCHAR      WorkLoadData[MQ_EXIT_DATA_LENGTH];              /* [v51] V51 */
   MQLONG      WorkLoadLength;                                 /* [v51] V51 */
   MQCHAR      CreationDate[MQ_CREATION_DATE_LENGTH];          /*[v51] V51 */
   MQCHAR      CreationTime[MQ_CREATION_TIME_LENGTH];          /*[v51] V51 */
   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];        /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];        /*[v51] V51 */
#if defined (MQ_OBJECT_NAME_LENGTH)
   MQCHAR      RepositoryName[MQ_OBJECT_NAME_LENGTH];          /* [v51] V51 */
   MQCHAR      RepositoryNameList[MQ_OBJECT_NAME_LENGTH];      /* [v51] V51 */
   MQCHAR      QMgrId[MQ_Q_MGR_IDENTIFIER_LENGTH];             /*[v51] V51 */
#endif

#if defined (MQ_SSL_KEY_REPOSITORY_LENGTH)
   MQCHAR      SslCrlNameList[MQ_OBJECT_NAME_LENGTH];               /*[v53] V53 */
   MQCHAR      SslKeyRepos[MQ_SSL_KEY_REPOSITORY_LENGTH];               /*[v53] V53 */
   MQCHAR      SslCryptoHW[MQ_SSL_CRYPTO_HARDWARE_LENGTH];               /*[v53] V53 */
#endif
} QMgrParms;



typedef struct QParms {
   MQCHAR      QName[MQ_Q_NAME_LENGTH];
   MQLONG      QType;
   MQCHAR      QDesc[MQ_Q_DESC_LENGTH];
   MQLONG      InhibitGet;
   MQLONG      InhibitPut;
   MQLONG      DefPriority;
   MQLONG      DefPersistence;
   MQCHAR      ProcessName[MQ_PROCESS_NAME_LENGTH];
   MQLONG      MaxQDepth;
   MQLONG      MaxMsgLength;
   MQLONG      BackoutThreshold;
   MQCHAR      BackoutReqQName[MQ_Q_NAME_LENGTH];
   MQLONG      Shareability;
   MQLONG      DefInputOpenOption;
   MQLONG      HardenGetBackout;
   MQLONG      MsgDeliverySequence;
   MQLONG      RetentionInterval;
   MQLONG      DefinitionType;
   MQLONG      Usage;
   MQLONG      OpenInputCount;
   MQLONG      OpenOutputCount;
   MQLONG      CurrentQDepth;
   MQCHAR      CreationDate[MQ_CREATION_DATE_LENGTH];
   MQCHAR      CreationTime[MQ_CREATION_TIME_LENGTH];
   MQCHAR      InitiationQName[MQ_Q_NAME_LENGTH];
   MQLONG      TriggerControl;
   MQLONG      TriggerType;
   MQLONG      TriggerMsgPriority;
   MQLONG      TriggerDepth;
   MQCHAR      TriggerData[MQ_TRIGGER_DATA_LENGTH];
   MQCHAR      BaseQName[MQ_Q_NAME_LENGTH];
   MQCHAR      RemoteQName[MQ_Q_NAME_LENGTH];
   MQCHAR      RemoteQMgrName[MQ_Q_NAME_LENGTH];
   MQCHAR      XmitQName[MQ_Q_NAME_LENGTH];
   MQLONG      Scope;
   MQLONG      QDepthHighLimit;
   MQLONG      QDepthLowLimit;
   MQLONG      QDepthMaxEvent;
   MQLONG      QDepthHighEvent;
   MQLONG      QDepthLowEvent;
   MQLONG      QServiceInterval;
   MQLONG      QServiceIntervalEvent;

   MQLONG      DistLists;    /*[wms] V5 */

   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];   /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];   /*[v51] V51 */
#if defined (MQ_OBJECT_NAME_LENGTH)
   MQCHAR      ClusterName[MQ_CLUSTER_NAME_LENGTH];    /*[v51] V51 */
   MQCHAR      ClusterNameList[MQ_OBJECT_NAME_LENGTH];    /*[v51] V51 */
   MQLONG      DefBind;                                   /*[v51] V51 */
#endif
} QParms;


typedef struct ChlParms {
   MQCHAR      ChannelName[MQ_CHANNEL_NAME_LENGTH];
   MQLONG      ChannelType;
   MQLONG      TransportType;
   MQCHAR      ModeName[MQ_MODE_NAME_LENGTH];
   MQCHAR      TpName[MQ_TP_NAME_LENGTH];
   MQCHAR      QMgrName[MQ_Q_MGR_NAME_LENGTH];
   MQCHAR      XmitQName[MQ_Q_NAME_LENGTH];
   MQCHAR      ConnectionName[MQ_CONN_NAME_LENGTH];
   MQCHAR      MCAName[MQ_MCA_NAME_LENGTH];
   MQCHAR      ChannelDesc[MQ_CHANNEL_DESC_LENGTH];
   MQLONG      BatchSize;
   MQLONG      DiscInterval;
   MQLONG      ShortRetryCount;
   MQLONG      ShortRetryInterval;
   MQLONG      LongRetryCount;
   MQLONG      LongRetryInterval;
   MQLONG      DataConversion;
   MQCHAR      SecurityExit[MQ_EXIT_NAME_LENGTH];

   /* [v530] Flags to tell if the following attributes are only strings */
   /* or lists. Ie- lists are used for chaining and a */
   /* value of 1 indicates list */
   MQLONG      MsgExitListFlag;
   MQLONG      SendExitListFlag;
   MQLONG      ReceiveExitListFlag;
   MQLONG      MsgUserDataListFlag;
   MQLONG      SendUserDataListFlag;
   MQLONG      ReceiveUserDataListFlag;
   MQLONG      MsgRetryUserDataListFlag;
   MQLONG      MsgRetryExitListFlag;

   /* [v512] becuase we actually store this as 'exit1','exit2','exit3' */
   /* we double the size to allow for the quotes and commas            */
   MQCHAR      MsgExit[MQ_TOTAL_EXIT_NAME_LENGTH*2];
   MQCHAR      SendExit[MQ_TOTAL_EXIT_NAME_LENGTH*2];
   MQCHAR      ReceiveExit[MQ_TOTAL_EXIT_NAME_LENGTH*2];
   MQCHAR      MsgUserData[MQ_TOTAL_EXIT_DATA_LENGTH*2];
   MQCHAR      SendUserData[MQ_TOTAL_EXIT_DATA_LENGTH*2];
   MQCHAR      ReceiveUserData[MQ_TOTAL_EXIT_DATA_LENGTH*2];
   MQCHAR      MsgRetryExit[MQ_TOTAL_EXIT_NAME_LENGTH*2];
   MQCHAR      MsgRetryUserData[MQ_TOTAL_EXIT_DATA_LENGTH*2];

   MQLONG      PutAuthority;
   MQLONG      SeqNumberWrap;
   MQLONG      MaxMsgLength;
   MQCHAR      SecurityUserData[MQ_EXIT_DATA_LENGTH];
   MQLONG      MCAType;
   MQCHAR      MCAUserIdentifier[XZMCAUSERLEN];
   MQCHAR      UserIdentifier[MQ_USER_ID_LENGTH];
   MQCHAR      Password[MQ_PASSWORD_LENGTH];

   MQLONG      MsgRetryCount;                         /*[wms]  */
   MQLONG      MsgRetryInterval;                      /*[wms]  */

   MQLONG      BatchInterval;                         /*[wms]V5*/
   MQLONG      HeartbeatInterval;                     /*[wms]V5*/
   MQLONG      NonPersistentMsgSpeed;                 /*[wms]V5*/

   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];   /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];   /*[v51] V51 */
#if defined (MQ_OBJECT_NAME_LENGTH)
   MQCHAR      ClusterName[MQ_CLUSTER_NAME_LENGTH];       /*[v51] V51 */
   MQCHAR      ClusterNameList[MQ_OBJECT_NAME_LENGTH];    /*[v51] V51 */
   MQLONG      NetworkPriority;                           /* [v51] v51 */
#endif
   MQLONG     InhibitAutostart;                     /* [v520] tandem nsk */


#if defined (MQCACH_LOCAL_ADDRESS)                  /* [v530] attributes */
   MQLONG       BatchHB;                       /* Batch Heartbeat */
   MQLONG       SslCAuth;                      /* Client Auth Required */
   MQCHAR       LocalAddr[MQ_LOCAL_ADDRESS_LENGTH];
   MQCHAR       SslCipher[MQ_SSL_CIPHER_SPEC_LENGTH];
   MQCHAR       SslPeer[MQ_SSL_PEER_NAME_LENGTH];

#endif
   MQLONG       KAInt;                      /* [v531] KAINT */

} ChlParms;


typedef struct ProcParms {
   MQCHAR      ProcessName[MQ_PROCESS_NAME_LENGTH];
   MQCHAR      ProcessDesc[MQ_PROCESS_DESC_LENGTH];
   MQLONG      ApplType;
   MQCHAR      ApplID[MQ_PROCESS_APPL_ID_LENGTH];
   MQCHAR      EnvData[MQ_PROCESS_ENV_DATA_LENGTH];
   MQCHAR      UserData[MQ_PROCESS_USER_DATA_LENGTH];

   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];   /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];   /*[v51] V51 */
} ProcParms;

#if defined (MQ_OBJECT_NAME_LENGTH)
   /* [v512] see the above comment                                   */
#define NAMELIST_BUFFER_SIZE ( MQNC_MAX_NAMELIST_NAME_COUNT*MQ_OBJECT_NAME_LENGTH * 2 )
typedef struct NameLParms {
   MQCHAR      NamelistName[MQ_NAMELIST_NAME_LENGTH];
   MQCHAR      NamelistDesc[MQ_NAMELIST_DESC_LENGTH];

   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];   /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];   /*[v51] V51 */
   MQLONG      NameCount;
  MQCHAR      Names[NAMELIST_BUFFER_SIZE];
} NameLParms;
#endif

/* v530 Authinfo */

#if defined (MQ_AUTH_INFO_NAME_LENGTH)
typedef struct AuthInfoParms {
   MQCHAR      AuthInfoName[MQ_AUTH_INFO_NAME_LENGTH];
   MQCHAR      AuthInfoDesc[MQ_AUTH_INFO_DESC_LENGTH];

   MQCHAR      AlterationDate[MQ_CREATION_DATE_LENGTH];   /*[v51] V51 */
   MQCHAR      AlterationTime[MQ_CREATION_TIME_LENGTH];   /*[v51] V51 */
   MQCHAR      ConnName[MQ_AUTH_INFO_CONN_NAME_LENGTH];
   MQCHAR      Password[MQ_LDAP_PASSWORD_LENGTH];
   MQLONG       AuthInfoType;
   MQCHAR       UserName[1024 /*MQ_LDAP_USER_NAME_LENGTH*/];
} AuthInfoParms;
#endif



MQLONG InquireDefinition( MQHCONN   hConn
                        , MQHOBJ    hAdminQ
                        , MQCHAR48  ReplyToQ
                        , MQLONG    SearchType /* QUEUE|CHANNEL|PROCESS|QMGR */
                        , MQLONG    SearchParm
                        , MQLONG    SearchParmLen
                        , MQCHAR   *SearchString
                        , MQLONG    SearchStringLen
                      );

MQLONG ProcessReplyQ( MQHCONN   hConn
                  , MQHOBJ    hReplyQ
                  , int *     pUserVersion
                  , int *     pQmgrVersion
                  , FILE **   outputfp /* v51 */
                  , char *    fileName
                  , MQLONG    ObjectSets
                  , int       SysObjs
                  );


/* ****************** */
/* Defined in QUEUE.C */
/* ****************** */

void ProcessQueueStringParm( MQCFST *pPCFString
                           , QParms *QDefn
                           , MQLONG    *iCC
                           );

void ProcessQueueIntegerParm( MQCFIN *pPCFInteger
                            , QParms *QDefn
                            , MQLONG    *iCC
                            );

void AddToFileQUEUE( QParms *QDefn,
                     int QmgrVersion,
                     char *fileName,
                     FILE *poutputfp, /* v51 */
                     MQLONG *iCC );



/* ******************** */
/* Defined in CHANNEL.C */
/* ******************** */

void ProcessChannelStringParm( MQCFST   *pPCFString
                             , ChlParms *ChlDefn
                             , MQLONG      *iCC
                             );

void ProcessChannelStringListParm( MQCFSL   *pPCFString
                             , ChlParms *ChlDefn
                             , MQLONG      *iCC
                             );

void ProcessChannelIntegerParm( MQCFIN   *pPCFInteger
                              , ChlParms *ChlDefn
                              , MQLONG      *iCC
                              );

void AddToFileCHANNEL( ChlParms *ChlDefn,
                       int QmgrVersion,
                       char * fileName,
                       FILE *poutputfp, /* v51 */
                       MQLONG *iCC );



/* ******************** */
/* Defined in PROCESS.C */
/* ******************** */

void ProcessProcStringParm( MQCFST    *pPCFString
                          , ProcParms *ProcDefn
                          , MQLONG       *iCC
                          );

void ProcessProcIntegerParm( MQCFIN    *pPCFInteger
                           , ProcParms *ProcDefn
                           , MQLONG       *iCC
                           );

void AddToFilePROCESS( ProcParms *ProcDefn,
                       int QmgrVersion,
                       char *fileName,
                       FILE *poutputfp, /* v51 */
                       MQLONG *iCC );



/* ***************** */
/* Defined in QMGR.C */
/* ***************** */

void ProcessQMgrStringParm( MQCFST    *pPCFString
                          , QMgrParms *QMgrDefn
                          , MQLONG       *iCC
                          );

void ProcessQMgrIntegerParm( MQCFIN    *pPCFInteger
                          , QMgrParms *QMgrDefn
                          , MQLONG       *iCC
                          );

void AddToFileQMGR( QMgrParms *QMgrDefn,
                    int QmgrVersion,
                    char *fileName,
                    FILE *poutputfp, /* v51 */
                    MQLONG *iCC );



#if defined (MQ_OBJECT_NAME_LENGTH)
/* ********************* */
/* Defined in NAMELIST.C */
/* ********************* */

void ProcessNamelistStringParm( MQCFST    *pPCFString
                          , NameLParms *NlDefn
                          , MQLONG       *iCC
                          );

void ProcessNamelistStringListParm( MQCFSL    *pPCFString
                          , NameLParms *NlDefn
                          , MQLONG       *iCC
                          );

void ProcessNamelistIntegerParm( MQCFIN    *pPCFInteger
                          , NameLParms *NlDefn
                          , MQLONG       *iCC
                          );

void AddToFileNAMELIST( NameLParms *NlDefn,
                        int QmgrVersion,
                        char *fileName,
                        FILE *poutputfp, /* v51 */
                        MQLONG *iCC  );

#endif


#if defined (MQ_AUTH_INFO_NAME_LENGTH)
/* ********************* */
/* Defined in authinfo.C */
/* ********************* */

void ProcessAuthInfoStringParm( MQCFST    *pPCFString
                          , AuthInfoParms *pAuthInfoDefn
                          , MQLONG       *iCC
                          );

void ProcessAuthInfoIntegerParm( MQCFIN    *pPCFInteger
                          , AuthInfoParms *pAuthInfoDefn
                          , MQLONG       *iCC
                          );

void AddToFileAUTHINFO(
                        AuthInfoParms *pAuthInfoDefn,
                        int QmgrVersion,
                        char *fileName,
                        FILE *poutputfp,
                        MQLONG *iCC  );

#endif

/* ******************** */
/* Defined in MQUTILS.C */
/* ******************** */

void MQParmCpy( char *target, char *source, int length );

void MQsprintf( char    *ParmBuffer
              , char    *Format
              , char    *Insert
              , int      Length
              );

/* [V530] Switches for MQSprintf_X */
#define MQSPRINTF_X_NONE                0x00
#define MQSPRINTF_X_NO_TRAILING_BLANKS  0x01
#define MQSPRINTF_X_DOUBLE_QUOTES       0x02
#define MQSPRINTF_X_LINE_BREAKS         0x04

/* [v5.3.0] Extended MQSprintf which is controlled via switches */
void MQsprintf_X ( char    *ParmBuffer
                   , char    *Format
                   , char    *Insert
                   , int      Length
                   , int      Options
                   );

MQLONG PutMsg( MQHCONN   hConn       /* Connection to queue manager            */
             , MQCHAR8   MsgFormat   /* Format of user data to be put in msg   */
             , MQHOBJ    hQName      /* handle of queue to put the message to  */
             , MQCHAR48  QName       /* name of queue to put the message to    */
             , MQCHAR48  ReplyToQ    /* name of q on which to receive replies  */
             , MQBYTE   *UserMsg     /* The user data to be put in the message */
             , MQLONG    UserMsgLen  /*                                        */
             );

MQLONG GetMsg( MQHCONN   hConn           /* handle of queue manager            */
           , MQHOBJ    hQName          /* handle of queue to read from       */
           , MQBYTE  **UserMsg         /* Input/Output buffer containing msg */
           , MQLONG   *ReadBufferLen   /* Length of supplied buffer          */
           );

MQLONG OpenQ( MQHCONN     hConn
            , MQCHAR48    QMgrName
            , MQCHAR48    QName
            , MQCHAR48   *DynamicQName
            , MQLONG      OpenOpts
            , MQLONG     *CompCode
            );

void  makeStringFromList( char * ,
                          char *,
                          MQLONG,
                          MQLONG
                          );

/* Some constants for our own use .... */
#define WAIT_INT     60000           /* 1 minute */
#define INT_BUFSZE    2048            /* 2 K       */


/* Application return codes */
/* [OvE] On OpenVMS, switch WARNING and OK */

#if MQAT_DEFAULT == MQAT_VMS
   #define  WARNING       0  /* we (probably) created output, but there were problems */
   #define  OK            1  /* everything worked, no problems encountered */
   #define  ERROR         2  /* nothing was created for output */
#else
   #define  OK            0  /* everything worked, no problems encountered */
   #define  WARNING       1  /* we (probably) created output, but there were problems */
   #define  ERROR         2  /* nothing was created for output */
#endif

/* os400 needs a special open mode */

#if MQAT_DEFAULT == MQAT_OS400
   #define FILENAMESIZE 33 /* v51 AvV Lib/File(Member) -> 10/10(10) */
   #define OPENMODE "a lrecl=128"
   #define DEFAULTOUTPUTFILE "*CURLIB/QMQSC(SAVEQMGR)"
   #define DEFAULTOUTPUTPREFIX "*CURLIB/QMQSC("
   #define DEFAULTOUTPUTPOSTFIX ")"
#else
   #define FILENAMESIZE 256
   #define OPENMODE "a"
   #define DEFAULTOUTPUTFILE "SAVEQMGR.TST"
   #define DEFAULTOUTPUTPREFIX ""
   #define DEFAULTOUTPUTPOSTFIX ".MQS"
#endif

#define PBSIZE 1025  /* Size of PrintBuffer */

